window.Gleam.initSearch([{"doc":"glimit","title":"glimit","content":"# glimit\n\n[![Package Version](https://img.shields.io/hexpm/v/glimit)](https://hex.pm/packages/glimit)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/glimit/)\n[![test](https://github.com/nootr/glimit/actions/workflows/test.yml/badge.svg)](https://github.com/nootr/glimit/actions/workflows/test.yml)\n\nA simple, framework-agnostic, in-memory rate limiter for Gleam. 💫\n\n&gt; ⚠️  This library is still in development, use at your own risk.\n\n\n## Features\n\n* ✨ Simple and easy to use.\n* 📏 Rate limits based on any key (e.g. IP address, or user ID).\n* 🪣 Uses a distributed Token Bucket algorithm to rate limit requests.\n* 🗄️ No back-end service needed; stores rate limit stats in-memory.\n\n\n## Usage\n\nA very minimalistic example of how to use `glimit` would be the following snippet:\n\n```gleam\nimport glimit\n\nlet limiter =\n  glimit.new()\n  |&gt; glimit.per_second(2)\n  |&gt; glimit.identifier(fn(x) { x })\n  |&gt; glimit.on_limit_exceeded(fn(_) { &quot;Stop!&quot; })\n\nlet func =\n  fn(_) { &quot;OK&quot; }\n  |&gt; glimit.apply(limiter)\n\nfunc(&quot;🚀&quot;) // &quot;OK&quot;\nfunc(&quot;💫&quot;) // &quot;OK&quot;\nfunc(&quot;💫&quot;) // &quot;OK&quot;\nfunc(&quot;💫&quot;) // &quot;Stop!&quot;\nfunc(&quot;🚀&quot;) // &quot;OK&quot;\nfunc(&quot;🚀&quot;) // &quot;Stop!&quot;\n```\n\nMore practical examples can be found in the `examples/` directory, such as Wisp and Mist server examples.\n\n\n## Constraints\n\nWhile the in-memory rate limiter is simple and easy to use, it does have an important constraint: it is scoped to the BEAM VM cluster it runs in. This means that if your application is running across multiple BEAM VM clusters, the rate limiter will not be shared between them.\n\nThere are plans to add support for a centralized data store using Redis in the future.\n\n\n## Documentation\n\nFurther documentation can be found at &lt;https://hexdocs.pm/glimit/glimit.html&gt;.\n\n\n## Contributing\n\nContributions like PR&#39;s, bug reports or suggestions are more than welcome! ♥️\n","url":"index.html"},{"doc":"glimit","title":"RateLimiter","content":"pub type RateLimiter(a, b, id) {\n  RateLimiter(\n    rate_limiter_registry: RateLimiterRegistryActor(id),\n    on_limit_exceeded: fn(a) -&gt; b,\n    identifier: fn(a) -&gt; id,\n  )\n}\n A rate limiter.\n\nRateLimiter(\n  rate_limiter_registry: RateLimiterRegistryActor(id),\n  on_limit_exceeded: fn(a) -&gt; b,\n  identifier: fn(a) -&gt; id,\n)\n\n\nSynonyms:\nglimit.RateLimiter\nglimit RateLimiter","url":"glimit.html#RateLimiter"},{"doc":"glimit","title":"RateLimiterBuilder","content":"pub type RateLimiterBuilder(a, b, id) {\n  RateLimiterBuilder(\n    per_second: Option(fn(id) -&gt; Int),\n    burst_limit: Option(fn(id) -&gt; Int),\n    identifier: Option(fn(a) -&gt; id),\n    on_limit_exceeded: Option(fn(a) -&gt; b),\n  )\n}\n A builder for configuring the rate limiter.\n\nRateLimiterBuilder(\n  per_second: Option(fn(id) -&gt; Int),\n  burst_limit: Option(fn(id) -&gt; Int),\n  identifier: Option(fn(a) -&gt; id),\n  on_limit_exceeded: Option(fn(a) -&gt; b),\n)\n\n\nSynonyms:\nglimit.RateLimiterBuilder\nglimit RateLimiterBuilder","url":"glimit.html#RateLimiterBuilder"},{"doc":"glimit","title":"apply","content":"pub fn apply(\n  func: fn(a) -&gt; b,\n  config: RateLimiterBuilder(a, b, c),\n) -&gt; fn(a) -&gt; b\n Apply the rate limiter to a request handler or function.\n\n Panics if the rate limiter registry cannot be started or if the `identifier`\n function or `on_limit_exceeded` function is missing.\n\nSynonyms:\nglimit.apply\nglimit apply","url":"glimit.html#apply"},{"doc":"glimit","title":"apply_built","content":"pub fn apply_built(\n  func: fn(a) -&gt; b,\n  limiter: RateLimiter(a, b, c),\n) -&gt; fn(a) -&gt; b\n Apply the rate limiter to a request handler or function.\n\n This function is useful if you want to build the rate limiter manually using the\n `build` function.\n\nSynonyms:\nglimit.apply_built\nglimit apply_built","url":"glimit.html#apply_built"},{"doc":"glimit","title":"build","content":"pub fn build(\n  config: RateLimiterBuilder(a, b, c),\n) -&gt; Result(RateLimiter(a, b, c), String)\n Build the rate limiter.\n\n Note that using `apply` will already build the rate limiter, so this function is\n only useful if you want to build the rate limiter manually and apply it to multiple\n functions.\n\n To apply the resulting rate limiter to a function or handler, use the `apply_built`\n function.\n\nSynonyms:\nglimit.build\nglimit build","url":"glimit.html#build"},{"doc":"glimit","title":"burst_limit","content":"pub fn burst_limit(\n  limiter: RateLimiterBuilder(a, b, c),\n  burst_limit: Int,\n) -&gt; RateLimiterBuilder(a, b, c)\n Set the maximum number of available tokens.\n\n The maximum number of available tokens is the maximum number of requests that can be\n made in a single second. The default value is the same as the rate limit per second.\n\n # Example\n\n \n import glimit\n\n let limiter =\n   glimit.new()\n   |&gt; glimit.per_second(10)\n   |&gt; glimit.burst_limit(100)\n \n\nSynonyms:\nglimit.burst_limit\nglimit burst_limit","url":"glimit.html#burst_limit"},{"doc":"glimit","title":"burst_limit_fn","content":"pub fn burst_limit_fn(\n  limiter: RateLimiterBuilder(a, b, c),\n  burst_limit_fn: fn(c) -&gt; Int,\n) -&gt; RateLimiterBuilder(a, b, c)\n Set the maximum number of available tokens, based on the identifier.\n\n # Example\n\n \n import glimit\n\n let limiter =\n   glimit.new()\n   |&gt; glimit.identifier(fn(request) { request.user_id })\n   |&gt; glimit.per_second(10)\n   |&gt; glimit.burst_limit_fn(fn(user_id) {\n     db.get_burst_limit(user_id)\n   })\n \n\nSynonyms:\nglimit.burst_limit_fn\nglimit burst_limit_fn","url":"glimit.html#burst_limit_fn"},{"doc":"glimit","title":"identifier","content":"pub fn identifier(\n  limiter: RateLimiterBuilder(a, b, c),\n  identifier: fn(a) -&gt; c,\n) -&gt; RateLimiterBuilder(a, b, c)\n Set the identifier function to be used to identify the rate limit.\n\n # Example\n\n \n import glimit\n\n let limiter =\n   glimit.new()\n   |&gt; glimit.identifier(fn(request) { request.ip })\n \n\nSynonyms:\nglimit.identifier\nglimit identifier","url":"glimit.html#identifier"},{"doc":"glimit","title":"new","content":"pub fn new() -&gt; RateLimiterBuilder(a, b, c)\n Create a new rate limiter builder.\n\nSynonyms:\nglimit.new\nglimit new","url":"glimit.html#new"},{"doc":"glimit","title":"on_limit_exceeded","content":"pub fn on_limit_exceeded(\n  limiter: RateLimiterBuilder(a, b, c),\n  on_limit_exceeded: fn(a) -&gt; b,\n) -&gt; RateLimiterBuilder(a, b, c)\n Set the handler to be called when the rate limit is reached.\n\n # Example\n\n \n import glimit\n\n let limiter =\n   glimit.new()\n   |&gt; glimit.per_second(10)\n   |&gt; glimit.on_limit_exceeded(fn(_request) { &quot;Rate limit reached&quot; })\n \n\nSynonyms:\nglimit.on_limit_exceeded\nglimit on_limit_exceeded","url":"glimit.html#on_limit_exceeded"},{"doc":"glimit","title":"per_second","content":"pub fn per_second(\n  limiter: RateLimiterBuilder(a, b, c),\n  limit: Int,\n) -&gt; RateLimiterBuilder(a, b, c)\n Set the rate of new available tokens per second.\n\n Note that this is not the maximum number of requests that can be made in a single\n second, but the rate at which tokens are added to the bucket. Think of this as the\n steady state rate limit, while the `burst_limit` function sets the maximum number of\n available tokens (or the burst rate limit).\n\n This value is also used as the default value for the `burst_limit` function.\n\n # Example\n\n \n import glimit\n\n let limiter =\n   glimit.new()\n   |&gt; glimit.per_second(10)\n \n\nSynonyms:\nglimit.per_second\nglimit per_second","url":"glimit.html#per_second"},{"doc":"glimit","title":"per_second_fn","content":"pub fn per_second_fn(\n  limiter: RateLimiterBuilder(a, b, c),\n  limit_fn: fn(c) -&gt; Int,\n) -&gt; RateLimiterBuilder(a, b, c)\n Set the rate limit per second, based on the identifier.\n\n # Example\n\n \n import glimit\n\n let limiter =\n   glimit.new()\n   |&gt; glimit.identifier(fn(request) { request.user_id })\n   |&gt; glimit.per_second_fn(fn(user_id) {\n     db.get_rate_limit(user_id)\n   })\n \n\nSynonyms:\nglimit.per_second_fn\nglimit per_second_fn","url":"glimit.html#per_second_fn"},{"doc":"glimit","title":"glimit","content":" This module provides a distributed rate limiter that can be used to limit the\n number of requests or function calls per second for a given identifier.\n\n A single actor is used to assign one rate limiter actor per identifier. The\n rate limiter actor then uses a Token Bucket algorithm to determine if a\n request or function call should be allowed to proceed. A separate process is\n polling the rate limiters to remove full buckets to reduce unnecessary memory\n usage.\n\n The rate limits are configured using the following two options:\n\n - `per_second`: The rate of new available tokens per second. Think of this\n   as the steady state rate limit.\n - `burst_limit`: The maximum number of available tokens. Think of this as\n   the burst rate limit. The default value is the `per_second` rate limit.\n\n The rate limiter can be applied to a function or handler using the `apply`\n function, which returns a new function that checks the rate limit before\n calling the original function.\n\n # Example\n\n ```gleam\n import glimit\n\n let limiter =\n   glimit.new()\n   |&gt; glimit.per_second(10)\n   |&gt; glimit.burst_limit(100)\n   |&gt; glimit.identifier(fn(request) { request.ip })\n   |&gt; glimit.on_limit_exceeded(fn(_request) { &quot;Rate limit reached&quot; })\n\n let handler =\n   fn(_request) { &quot;Hello, world!&quot; }\n   |&gt; glimit.apply(limiter)\n ```\n","url":"glimit.html"}]);